@@ -56,7 +56,7 @@ options configure.cxx \
 
 default configure.cxx                   {[portconfigure::configure_get_compiler cxx]}
 default configure.cxx_archflags         {[portconfigure::configure_get_archflags cxx]}
-default configure.cxx_stdlib            {$cxx_stdlib}
+default configure.cxx_stdlib            {[portconfigure::configure_get_cxx_stdlib]}
 default configure.cxxflags \
         {[portconfigure::construct_cxxflags ${configure.optflags}]}
 default configure.objcxx                {[portconfigure::configure_get_compiler objcxx]}
@@ -98,6 +98,14 @@ proc portconfigure::stdlib_trace {opt action args} {
     }
     return
 }
+proc portconfigure::configure_get_cxx_stdlib {} {
+    global cxx_stdlib configure.cxx
+    if {![string match *g++-mp-* ${configure.cxx}]} {
+        return $cxx_stdlib
+    } else {
+        return macports-libstdc++
+    }
+}
 
 # ********** END C++ / OBJECTIVE-C++ **********
 
@@ -385,23 +393,58 @@ proc portconfigure::configure_get_ld_archflags {} {
 
 proc portconfigure::configure_get_sdkroot {sdk_version} {
     global developer_dir macosx_version xcodeversion os.arch os.platform
-    if {${os.platform} eq "darwin" && ($sdk_version ne $macosx_version
-        || (${os.arch} eq "powerpc" && $macosx_version eq "10.4" && [variant_exists universal] && [variant_isset universal]))} {
-        if {[vercmp $xcodeversion 4.3] < 0} {
-            set sdks_dir ${developer_dir}/SDKs
-        } else {
-            set sdks_dir ${developer_dir}/Platforms/MacOSX.platform/Developer/SDKs
-        }
-        if {$sdk_version eq "10.4"} {
-            set sdk ${sdks_dir}/MacOSX10.4u.sdk
-        } else {
-            set sdk ${sdks_dir}/MacOSX${sdk_version}.sdk
-        }
-        if {[file exists $sdk]} {
-            return $sdk
-        }
+
+    # This is only relevant for macOS
+    if {${os.platform} ne "darwin"} {
+        return {}
     }
-    return {}
+
+    # Special hack for Tiger/ppc, since the system libraries do not contain intel slices
+    if {${os.arch} eq "powerpc" && $macosx_version eq "10.4" && [variant_exists universal] && [variant_isset universal]} {
+        return ${developer_dir}/SDKs/MacOSX10.4u.sdk
+    }
+
+    # Use the DevSDK (eg: /usr/include) if present and the requested SDK version matches the host version
+    if {$sdk_version eq $macosx_version && [file exists /usr/include]} {
+        return {}
+    }
+
+    if {[vercmp $xcodeversion 4.3] < 0} {
+        set sdks_dir ${developer_dir}/SDKs
+    } else {
+        set sdks_dir ${developer_dir}/Platforms/MacOSX.platform/Developer/SDKs
+    }
+
+    if {$sdk_version eq "10.4"} {
+        set sdk ${sdks_dir}/MacOSX10.4u.sdk
+    } else {
+        set sdk ${sdks_dir}/MacOSX${sdk_version}.sdk
+    }
+
+    if {[file exists $sdk]} {
+        return $sdk
+    }
+
+    if {![catch {set sdk [exec xcrun --sdk macosx${sdk_version} --show-sdk-path 2> /dev/null]}]} {
+        return $sdk
+    }
+
+    set sdk /Library/Developer/CommandLineTools/SDKs/MacOSX${sdk_version}.sdk
+    if {[file exists $sdk]} {
+        return $sdk
+    }
+
+    # TODO: Support falling back to "macosx" if it is present?
+    #       This leads to problems when it is newer than the base OS because many OSS assume that
+    #       the SDK version matches the deployment target, so they unconditionally try to use
+    #       symbols that are only available on newer OS versions..
+    #if {![catch {set sdk [exec xcrun --sdk macosx --show-sdk-path 2> /dev/null]}]} {
+    #    ui_warn "Unable to determine location of the macOS ${sdk_version} SDK.  Using the default macOS SDK."
+    #    return $sdk
+    #}
+
+    ui_error "Unable to determine location of a macOS SDK."
+    return -code error "Unable to determine location of a macOS SDK."
 }
 
 # internal function to determine the "-arch xy" flags for the compiler
@@ -480,7 +523,9 @@ proc portconfigure::configure_get_default_compiler {} {
 
 # internal function to choose compiler fallback list based on platform
 proc portconfigure::get_compiler_fallback {} {
-    global xcodeversion macosx_deployment_target default_compilers configure.sdkroot configure.cxx_stdlib os.major
+    global xcodeversion macosx_deployment_target default_compilers \
+           configure.sdkroot configure.cxx_stdlib cxx_stdlib os.major \
+           option_defaults
 
     # Check our override
     if {[info exists default_compilers]} {
@@ -499,6 +544,7 @@ proc portconfigure::get_compiler_fallback {} {
             if {[string match *10.4u* ${configure.sdkroot}]} {
                 return {gcc-4.0}
             }
+            # No return here. 3.2.x with newer SDKs than 10.4u is handled below.
         } elseif {[vercmp $xcodeversion 3.0] >= 0} {
             if {"ppc" in $canonical_archs || "ppc64" in $canonical_archs} {
                 return {gcc-4.2 apple-gcc-4.2 gcc-4.0 macports-gcc-6 macports-gcc-7}
@@ -532,13 +578,23 @@ proc portconfigure::get_compiler_fallback {} {
     } elseif {[vercmp $xcodeversion 4.0] >= 0} {
         lappend compilers llvm-gcc-4.2 clang
     } else {
+        # 3.2.x
         lappend compilers gcc-4.2 clang llvm-gcc-4.2
     }
 
     # Determine which versions of clang we prefer
-    if {${configure.cxx_stdlib} eq "libc++"} {
+    # There is a recursion trap here: the default value of configure.cxx_stdlib
+    # is determined by a proc that may end up calling us to find out which
+    # compiler is being used. So, bypass that if the option hasn't already
+    # been set to a particular value.
+    if {![info exists option_defaults(configure.cxx_stdlib)]} {
+        set our_stdlib ${configure.cxx_stdlib}
+    } else {
+        set our_stdlib $cxx_stdlib
+    }
+    if {$our_stdlib eq "libc++"} {
         # clang-3.5+ require libc++
-        lappend compilers macports-clang-4.0
+        lappend compilers macports-clang-5.0 macports-clang-4.0
 
         if {${os.major} < 17} {
             # The High Sierra SDK requires a toolchain that can apply nullability to uuid_t
@@ -736,8 +792,7 @@ proc portconfigure::configure_main {args} {
            configure.perl configure.python configure.ruby configure.install configure.awk configure.bison \
            configure.pkg_config configure.pkg_config_path \
            configure.ccache configure.distcc configure.cpp configure.javac configure.sdkroot \
-           configure.march configure.mtune configure.cxx_stdlib \
-           os.platform os.major
+           configure.march configure.mtune os.platform os.major
     foreach tool {cc cxx objc objcxx f77 f90 fc ld} {
         global configure.${tool} configure.${tool}_archflags
     }
@@ -807,12 +862,19 @@ proc portconfigure::configure_main {args} {
             CC CXX OBJC OBJCXX FC F77 F90 JAVAC \
             CFLAGS CPPFLAGS CXXFLAGS OBJCFLAGS OBJCXXFLAGS \
             FFLAGS F90FLAGS FCFLAGS LDFLAGS LIBS CLASSPATH \
-            PERL PYTHON RUBY INSTALL AWK BISON PKG_CONFIG PKG_CONFIG_PATH \
+            PERL PYTHON RUBY INSTALL AWK BISON PKG_CONFIG \
         } {
             set value [option configure.[string tolower $env_var]]
             append_to_environment_value configure $env_var {*}$value
         }
 
+        foreach env_var { \
+            PKG_CONFIG_PATH \
+        } {
+            set value [option configure.[string tolower $env_var]]
+            append_to_environment_value configure $env_var [join $value ":"]
+        }
+
         # https://trac.macports.org/ticket/34221
         if {${os.platform} eq "darwin" && ${os.major} == 12} {
             append_to_environment_value configure "__CFPREFERENCES_AVOID_DAEMON" 1
