@@ -1,8 +1,7 @@
 # et:ts=4
 # portextract.tcl
-# $Id$
 #
-# Copyright (c) 2005, 2007-2011, 2013 The MacPorts Project
+# Copyright (c) 2005, 2007-2011, 2013-2014, 2016, 2018 The MacPorts Project
 # Copyright (c) 2002 - 2003 Apple Inc.
 # Copyright (c) 2007 Markus W. Weissmann <mww@macports.org>
 # All rights reserved.
@@ -65,7 +64,7 @@ set_ui_prefix
 # Helper function for portextract.tcl that strips all tag names from a list
 # Used to clean ${distfiles} for setting the ${extract.only} default
 proc portextract::disttagclean {list} {
-    if {"$list" == ""} {
+    if {$list eq ""} {
         return $list
     }
     foreach name $list {
@@ -75,7 +74,7 @@ proc portextract::disttagclean {list} {
 }
 
 proc portextract::extract_start {args} {
-    global UI_PREFIX extract.dir extract.mkdir use_bzip2 use_lzma use_xz use_zip use_7z use_lzip use_dmg
+    global UI_PREFIX extract.dir extract.mkdir use_tar use_bzip2 use_lzma use_xz use_zip use_7z use_lzip use_dmg
 
     ui_notice "$UI_PREFIX [format [msgcat::mc "Extracting %s"] [option subport]]"
 
@@ -89,8 +88,16 @@ proc portextract::extract_start {args} {
         file mkdir ${worksrcpath}
         set extract.dir ${worksrcpath}
     }
-    if {[tbool use_bzip2]} {
-        option extract.cmd [findBinary bzip2 ${portutil::autoconf::bzip2_path}]
+    if {[tbool use_tar]} {
+        option extract.cmd [findBinary tar ${portutil::autoconf::tar_command}]
+        option extract.pre_args -xf
+        option extract.post_args ""
+    } elseif {[tbool use_bzip2]} {
+        if {![catch {findBinary lbzip2} result]} {
+            option extract.cmd $result
+        } else {
+            option extract.cmd [findBinary bzip2 ${portutil::autoconf::bzip2_path}]
+        }
     } elseif {[tbool use_lzma]} {
         option extract.cmd [findBinary lzma ${portutil::autoconf::lzma_path}]
     } elseif {[tbool use_xz]} {
@@ -117,7 +124,7 @@ proc portextract::extract_start {args} {
 }
 
 proc portextract::extract_main {args} {
-    global UI_PREFIX filespath worksrcpath extract.dir usealtworkpath altprefix use_dmg
+    global UI_PREFIX filespath worksrcpath extract.dir use_dmg
 
     if {![exists distfiles] && ![exists extract.only]} {
         # nothing to do
@@ -128,8 +135,6 @@ proc portextract::extract_main {args} {
         ui_info "$UI_PREFIX [format [msgcat::mc "Extracting %s"] $distfile]"
         if {[file exists $filespath/$distfile]} {
             option extract.args "'$filespath/$distfile'"
-        } elseif {![file exists "[option distpath]/$distfile"] && !$usealtworkpath && [file exists "${altprefix}[option distpath]/$distfile"]} {
-            option extract.args "'${altprefix}[option distpath]/$distfile'"
         } else {
             option extract.args "'[option distpath]/$distfile'"
         }
@@ -149,9 +154,7 @@ proc portextract::extract_main {args} {
             return -code error "$result"
         }
 
-        # start gsoc08-privileges
         chownAsRoot ${extract.dir}
-        # end gsoc08-privileges
     }
     return 0
 }
