@@ -1,8 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portimage.tcl
-# $Id$
 #
-# Copyright (c) 2004-2005, 2007-2011 The MacPorts Project
+# Copyright (c) 2004-2005, 2007-2011, 2014 The MacPorts Project
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
 # Copyright (c) 2002 Apple Inc.
 # All rights reserved.
@@ -40,6 +39,8 @@ package require registry_util 2.0
 package require macports 1.0
 package require Pextlib 1.0
 
+package require Tclx
+
 set UI_PREFIX "--> "
 
 # Port Images are installations of the destroot of a port into a compressed
@@ -76,7 +77,7 @@ proc activate_composite {name {v ""} {optionslist ""}} {
 
 # Activate a "Port Image"
 proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
-    global macports::prefix macports::registry.path registry_open UI_PREFIX
+    global macports::registry.path registry_open UI_PREFIX
     array set options $optionslist
     variable force
     variable noexec
@@ -87,6 +88,10 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
     if {[info exists options(ports_activate_no-exec)]} {
         set noexec $options(ports_activate_no-exec)
     }
+    set rename_list {}
+    if {[info exists options(portactivate_rename_files)]} {
+        set rename_list $options(portactivate_rename_files)
+    }
     if {![info exists registry_open]} {
         registry::open [::file join ${macports::registry.path} registry registry.db]
         set registry_open yes
@@ -131,8 +136,7 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
 
     ui_msg "$UI_PREFIX [format [msgcat::mc "Activating %s @%s"] $name $specifier]"
 
-    _activate_contents $requested
-    $requested state installed
+    _activate_contents $requested $rename_list
 }
 
 # takes a composite version spec rather than separate version,revision,variants
@@ -175,8 +179,8 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
     set name [$requested name]
     set specifier "[$requested version]_[$requested revision][$requested variants]"
 
-    if {$version ne "" && ($version != [$requested version] ||
-        ($revision ne "" && ($revision != [$requested revision] || $variants != [$requested variants])))} {
+    if {$version ne "" && ($version ne [$requested version] ||
+        ($revision ne "" && ($revision != [$requested revision] || $variants ne [$requested variants])))} {
         set v $version
         if {$revision ne ""} {
             append v _${revision}${variants}
@@ -184,8 +188,6 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
         return -code error "Active version of $name is not $v but ${specifier}."
     }
 
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Deactivating %s @%s"] $name $specifier]"
-
     if { [$requested installtype] ne "image" } {
         return -code error "Image error: ${name} @${specifier} not installed as an image."
     }
@@ -193,13 +195,17 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
     if { [$requested state] ne "installed" } {
         return -code error "Image error: ${name} @${specifier} is not active."
     }
-
+	
     if {![info exists options(ports_nodepcheck)] || ![string is true -strict $options(ports_nodepcheck)]} {
-        registry::check_dependents $requested $force "deactivate"
+        set retvalue [registry::check_dependents $requested $force "deactivate"]
+        if {$retvalue eq "quit"} {
+            return
+        }
     }
 
+    ui_msg "$UI_PREFIX [format [msgcat::mc "Deactivating %s @%s"] $name $specifier]"
+	
     _deactivate_contents $requested [$requested files] $force
-    $requested state imaged
 }
 
 proc _check_registry {name version revision variants} {
@@ -217,20 +223,29 @@ proc _check_registry {name version revision variants} {
             append composite_spec _${revision}${variants}
         }
     }
-    set ilist [eval registry::entry imaged $searchkeys]
+    set ilist [registry::entry imaged {*}$searchkeys]
 
     if { [llength $ilist] > 1 } {
-        ui_msg "$UI_PREFIX [msgcat::mc "The following versions of $name are currently installed:"]"
+        set portilist {}
+        set msg "The following versions of $name are currently installed:"
+        if {[macports::ui_isset ports_noninteractive]} {
+            ui_msg "$UI_PREFIX [msgcat::mc $msg]"
+        }
         foreach i $ilist {
-            set iname [$i name]
-            set iversion [$i version]
-            set irevision [$i revision]
-            set ivariants [$i variants]
-            if { [$i state] eq "installed" } {
-                ui_msg "$UI_PREFIX [format [msgcat::mc "    %s @%s_%s%s (active)"] $iname $iversion $irevision $ivariants]"
-            } else {
-                ui_msg "$UI_PREFIX [format [msgcat::mc "    %s @%s_%s%s"] $iname $iversion $irevision $ivariants]"
+            set portstr [format "%s @%s_%s%s" [$i name] [$i version] [$i revision] [$i variants]]
+            if {[$i state] eq "installed"} {
+                append portstr [msgcat::mc " (active)"]
             }
+
+            if {[info exists macports::ui_options(questions_singlechoice)]} {
+                lappend portilist "$portstr"
+            } else {
+                ui_msg "$UI_PREFIX     $portstr"
+            }
+        }
+        if {[info exists macports::ui_options(questions_singlechoice)]} {
+            set retvalue [$macports::ui_options(questions_singlechoice) $msg "Choice_Q1" $portilist]
+            return [lindex $ilist $retvalue]
         }
         throw registry::invalid "Registry error: Please specify the full version as recorded in the port registry."
     } elseif { [llength $ilist] == 1 } {
@@ -268,10 +283,10 @@ proc _activate_file {srcfile dstfile} {
                 ::file mkdir $dstfile
                 # fix attributes on the directory.
                 if {[getuid] == 0} {
-                    eval ::file attributes {$dstfile} [::file attributes $srcfile]
+                    ::file attributes $dstfile {*}[::file attributes $srcfile]
                 } else {
                     # not root, so can't set owner/group
-                    eval ::file attributes {$dstfile} -permissions [::file attributes $srcfile -permissions]
+                    ::file attributes $dstfile -permissions {*}[::file attributes $srcfile -permissions]
                 }
                 # set mtime on installed element
                 ::file mtime $dstfile [::file mtime $srcfile]
@@ -289,7 +304,6 @@ proc _activate_file {srcfile dstfile} {
 # extract an archive to a temporary location
 # returns: path to the extracted directory
 proc extract_archive_to_tmpdir {location} {
-    global macports::registry.path
     set extractdir [mkdtemp [::file dirname $location]/mpextractXXXXXXXX]
     set startpwd [pwd]
 
@@ -336,40 +350,58 @@ proc extract_archive_to_tmpdir {location} {
                 }
             }
             t(ar|bz|lz|xz|gz) {
-                set tar "tar"
-                if {[catch {set tar [macports::findBinary $tar ${macports::autoconf::tar_path}]} errmsg] == 0} {
+                global macports::hfscompression
+                # Opportunistic HFS compression. bsdtar will automatically
+                # disable this if filesystem does not support compression.
+                # Have to be root due to <https://trac.macports.org/ticket/56563>.
+                if {${macports::hfscompression} && [getuid] == 0 &&
+                        ![catch {macports::binaryInPath bsdtar}] &&
+                        ![catch {exec bsdtar -x --hfsCompression < /dev/null >& /dev/null}]} {
+                    ui_debug "Using bsdtar with HFS+ compression (if valid)"
+                    set unarchive.cmd "bsdtar"
+                    set unarchive.pre_args {-xvp --hfsCompression -f}
+                } else {
+                    set tar "tar"
+                    if {[catch {set tar [macports::findBinary $tar ${macports::autoconf::tar_path}]} errmsg]} {
+                        ui_debug $errmsg
+                        throw MACPORTS "No '$tar' was found on this system!"
+                    }
                     ui_debug "Using $tar"
                     set unarchive.cmd "$tar"
                     set unarchive.pre_args {-xvpf}
-                    if {[regexp {z2?$} ${unarchive.type}]} {
-                        set unarchive.args {-}
-                        if {[regexp {bz2?$} ${unarchive.type}]} {
+                }
+
+                if {[regexp {z2?$} ${unarchive.type}]} {
+                    set unarchive.args {-}
+                    if {[regexp {bz2?$} ${unarchive.type}]} {
+                        if {![catch {macports::binaryInPath lbzip2}]} {
+                            set gzip "lbzip2"
+                        } elseif {![catch {macports::binaryInPath pbzip2}]} {
+                            set gzip "pbzip2"
+                        } else {
                             set gzip "bzip2"
-                        } elseif {[regexp {lz$} ${unarchive.type}]} {
-                            set gzip "lzma"
-                        } elseif {[regexp {xz$} ${unarchive.type}]} {
-                            set gzip "xz"
-                        } else {
-                            set gzip "gzip"
-                        }
-                        if {[info exists macports::autoconf::${gzip}_path]} {
-                            set hint [set macports::autoconf::${gzip}_path]
-                        } else {
-                            set hint ""
-                        }
-                        if {[catch {set gzip [macports::findBinary $gzip $hint]} errmsg] == 0} {
-                            ui_debug "Using $gzip"
-                            set unarchive.pipe_cmd "$gzip -d -c ${location} |"
-                        } else {
-                            ui_debug $errmsg
-                            throw MACPORTS "No '$gzip' was found on this system!"
                         }
+                    } elseif {[regexp {lz$} ${unarchive.type}]} {
+                        set gzip "lzma"
+                    } elseif {[regexp {xz$} ${unarchive.type}]} {
+                        set gzip "xz"
                     } else {
-                        set unarchive.args "${location}"
+                        set gzip "gzip"
+                    }
+                    if {[info exists macports::autoconf::${gzip}_path]} {
+                        set hint [set macports::autoconf::${gzip}_path]
+                    } else {
+                        set hint ""
+                    }
+                    if {[catch {set gzip [macports::findBinary $gzip $hint]} errmsg] == 0} {
+                        ui_debug "Using $gzip"
+                        set unarchive.pipe_cmd "$gzip -d -c ${location} |"
+                    } else {
+                        ui_debug $errmsg
+                        throw MACPORTS "No '$gzip' was found on this system!"
                     }
                 } else {
-                    ui_debug $errmsg
-                    throw MACPORTS "No '$tar' was found on this system!"
+                    set unarchive.args "${location}"
                 }
             }
             xar {
@@ -406,7 +438,7 @@ proc extract_archive_to_tmpdir {location} {
         }
         
         # and finally, reinvent command_exec
-        if {${unarchive.pipe_cmd} == ""} {
+        if {${unarchive.pipe_cmd} eq ""} {
             set cmdstring "${unarchive.cmd} ${unarchive.pre_args} ${unarchive.args}"
         } else {
             set cmdstring "${unarchive.pipe_cmd} ( ${unarchive.cmd} ${unarchive.pre_args} ${unarchive.args} )"
@@ -423,10 +455,9 @@ proc extract_archive_to_tmpdir {location} {
 }
 
 ## Activates the contents of a port
-proc _activate_contents {port {imagefiles {}} {location {}}} {
+proc _activate_contents {port {rename_list {}}} {
     variable force
     variable noexec
-    global macports::prefix
 
     set files [list]
     set baksuffix .mp_[clock seconds]
@@ -457,7 +488,7 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
 
                 set owner [registry::entry owner $file]
 
-                if {$owner != {} && $owner != $port} {
+                if {$owner ne {} && $owner ne $port} {
                     # deactivate conflicting port if it is replaced_by this one
                     set result [mportlookup [$owner name]]
                     array unset portinfo
@@ -480,7 +511,7 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                             ::file rename -force -- $file $bakfile
                             lappend backups $file
                         }
-                        if { $owner != {} } {
+                        if { $owner ne {} } {
                             $owner deactivate [list $file]
                             $owner activate [list $file] [list "${file}${baksuffix}"]
                         }
@@ -488,9 +519,9 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                         # if we're not forcing the activation, then we bail out if
                         # we find any files that already exist, or have entries in
                         # the registry
-                        if { $owner != {} && $owner != $port } {
+                        if { $owner ne {} && $owner ne $port } {
                             throw registry::image-error "Image error: $file is being used by the active [$owner name] port.  Please deactivate this port first, or use 'port -f activate [$port name]' to force the activation."
-                        } elseif { $owner == {} && ![catch {::file type $file}] } {
+                        } elseif { $owner eq {} && ![catch {::file type $file}] } {
                             throw registry::image-error "Image error: $file already exists and does not belong to a registered port.  Unable to activate port [$port name]. Use 'port -f activate [$port name]' to force the activation."
                         }
                     }
@@ -505,7 +536,7 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                 # we'll set the directory attributes properly for all
                 # directories.
                 set directory [::file dirname $file]
-                while { [lsearch -exact $files $directory] == -1 } {
+                while {$directory ni $files} {
                     lappend files $directory
                     set directory [::file dirname $directory]
                 }
@@ -528,6 +559,15 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
         # We don't have to do this as mentioned above, but it makes the
         # debug output of activate make more sense.
         set files [lsort -increasing -unique $files]
+        # handle files that are to be renamed
+        set confirmed_rename_list {}
+        foreach {src dest} $rename_list {
+            set index [lsearch -exact -sorted $files $src]
+            if {$index != -1} {
+                set files [lreplace $files $index $index]
+                lappend confirmed_rename_list $src $dest
+            }
+        }
         set rollback_filelist {}
 
         registry::write {
@@ -539,6 +579,31 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                         lappend rollback_filelist $file
                     }
                 }
+                foreach {src dest} $confirmed_rename_list {
+                    $port deactivate [list $src]
+                    $port activate [list $src] [list $dest]
+                    if {[_activate_file ${extracted_dir}${src} $dest] == 1} {
+                        lappend rollback_filelist $dest
+                    }
+                }
+
+                # Recording that the port has been activated should be done
+                # here so that this information cannot be inconsistent with the
+                # state of the files on disk.
+                $port state installed
+            } catch {{POSIX SIG SIGINT} eCode eMessage} {
+                # Pressing ^C will (often?) print "^C" to the terminal; send
+                # a linebreak so our message appears after that.
+                ui_msg ""
+                ui_msg "Control-C pressed, rolling back, please wait."
+                # can't do it here since we're already inside a transaction
+                set deactivate_this yes
+                throw
+            } catch {{POSIX SIG SIGTERM} eCode eMessage} {
+                ui_msg "SIGTERM received, rolling back, please wait."
+                # can't do it here since we're already inside a transaction
+                set deactivate_this yes
+                throw
             } catch {*} {
                 ui_debug "Activation failed, rolling back."
                 # can't do it here since we're already inside a transaction
@@ -547,22 +612,41 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
             }
         }
     } catch {*} {
-        # roll back activation of this port
-        if {[info exists deactivate_this]} {
-            _deactivate_contents $port $rollback_filelist yes yes
-        }
-        # if any errors occurred, move backed-up files back to their original
-        # locations, then rethrow the error. Transaction rollback will take care
-        # of this in the registry.
-        foreach file $backups {
-            ::file rename -force -- "${file}${baksuffix}" $file
-        }
-        # reactivate deactivated ports
-        foreach entry [array names todeactivate] {
-            if {[$entry state] eq "imaged" && ($noexec || ![registry::run_target $entry activate ""])} {
-                activate [$entry name] [$entry version] [$entry revision] [$entry variants] [list ports_activate_no-exec $noexec]
+        # This code must run to completion, or the installation might be left
+        # in an inconsistent state. We store the old signal handling state,
+        # block the critical signals and restore to the previous state instead
+        # of unblocking.
+        # Note that this still contains a race condition: A user could press ^C
+        # fast enough so that the second error arrives before the error is
+        # caught, re-thrown and re-caught here. As far as I can see, there's no
+        # easy way around this problem.
+        set osignals [signal get {TERM INT}]
+        try {
+            # Block signals to avoid inconsistiencies.
+            signal block {TERM INT}
+
+            # roll back activation of this port
+            if {[info exists deactivate_this]} {
+                _deactivate_contents $port $rollback_filelist yes yes
             }
+            # if any errors occurred, move backed-up files back to their original
+            # locations, then rethrow the error. Transaction rollback will take care
+            # of this in the registry.
+            foreach file $backups {
+                ::file rename -force -- "${file}${baksuffix}" $file
+            }
+            # reactivate deactivated ports
+            foreach entry [array names todeactivate] {
+                if {[$entry state] eq "imaged" && ($noexec || ![registry::run_target $entry activate ""])} {
+                    activate [$entry name] [$entry version] [$entry revision] [$entry variants] [list ports_activate_no-exec $noexec]
+                }
+            }
+        } finally {
+            # We've completed all critical operations, re-enable the TERM and
+            # INT signals.
+            signal set $osignals
         }
+
         # remove temp image dir
         ::file delete -force $extracted_dir
         throw
@@ -622,7 +706,7 @@ proc _deactivate_contents {port imagefiles {force 0} {rollback 0}} {
 
             # Split out the filename's subpaths and add them to the image list
             # as well.
-            while { [lsearch -exact $files $directory] == -1 } {
+            while {$directory ni $files} {
                 lappend files $directory
                 set directory [::file dirname $directory]
             }
@@ -636,18 +720,38 @@ proc _deactivate_contents {port imagefiles {force 0} {rollback 0}} {
     # are after their elements.
     set files [lsort -decreasing -unique $files]
 
-    # Remove all elements.
-    if {!$rollback} {
-        registry::write {
-            $port deactivate $imagefiles
+    # Avoid interruptions while removing the files and updating the database to
+    # prevent inconsistencies from forming between filesystem and database.
+    set osignals [signal get {TERM INT}]
+
+    try {
+        # Block the TERM and INT signals to avoid being interrupted. Note that
+        # they might already be block at this point because
+        # _deactivate_contents might be called during rollback of
+        # _activate_contents, but because we're storing the old signal state
+        # and returning to that instead of unblocking it doesn't matter.
+        signal block {TERM INT}
+
+        # Remove all elements.
+        if {!$rollback} {
+            registry::write {
+                $port deactivate $imagefiles
+                foreach file $files {
+                    _deactivate_file $file
+                }
+
+                # Update the port's state in the same transaction as the file
+                # delete operations.
+                $port state imaged
+            }
+        } else {
             foreach file $files {
                 _deactivate_file $file
             }
         }
-    } else {
-        foreach file $files {
-            _deactivate_file $file
-        }
+    } finally {
+        # restore the signal block state
+        signal set $osignals
     }
 }
 
